@@ -2,4 +2,4 @@ try:
     import copy
     from collections import KeysView, ValuesView
 except ModuleNotFoundError as e:
-    raise ModuleNotFoundError('Module not found\n' + e)
+    raise ModuleNotFoundError('Module not found\n' + str(e))
